@@ -1,6 +1,6 @@
 /* poppler-form.h: qt interface to poppler
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
- * Copyright (C) 2008, 2011, 2016, 2017, 2019, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2008, 2011, 2016, 2017, 2019, 2020, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2012, Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
  * Copyright (C) 2017, Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
@@ -28,7 +28,8 @@
 #ifndef _POPPLER_QT5_FORM_H_
 #define _POPPLER_QT5_FORM_H_
 
-#include <time.h>
+#include <memory>
+#include <ctime>
 #include <QtCore/QDateTime>
 #include <QtCore/QList>
 #include <QtCore/QRectF>
@@ -205,9 +206,9 @@ namespace Poppler {
 
     protected:
 	/// \cond PRIVATE
-	FormField(FormFieldData &dd);
+	FormField(std::unique_ptr<FormFieldData> dd);
 
-	FormFieldData *m_formData;
+	std::unique_ptr<FormFieldData> m_formData;
 	/// \endcond
 
     private:
@@ -235,7 +236,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldButton(DocumentData *doc, ::Page *p, ::FormWidgetButton *w);
 	/// \endcond
-	~FormFieldButton();
+	~FormFieldButton() override;
 
 	FormType type() const override;
 
@@ -306,7 +307,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldText(DocumentData *doc, ::Page *p, ::FormWidgetText *w);
 	/// \endcond
-	~FormFieldText();
+	~FormFieldText() override;
 
 	FormType type() const override;
 
@@ -395,7 +396,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldChoice(DocumentData *doc, ::Page *p, ::FormWidgetChoice *w);
 	/// \endcond
-	~FormFieldChoice();
+	~FormFieldChoice() override;
 
 	FormType type() const override;
 
@@ -409,6 +410,14 @@ namespace Poppler {
 	 */
 	QStringList choices() const;
 
+	/**
+	  The possible choices of the choice field.
+	  The first value of the pair is the display name of the choice,
+	  The second value is the export value (i.e. for use in javascript, etc) of the choice
+	  @since 0.87
+	 */
+	QVector<QPair<QString,QString>> choicesWithExportValues() const;
+
 	/**
 	  Whether this FormFieldChoice::ComboBox is editable, i.e. the user
 	  can type in a custom value.
@@ -741,7 +750,8 @@ namespace Poppler {
 	enum SignatureType {
 	    AdbePkcs7sha1,
 	    AdbePkcs7detached,
-	    EtsiCAdESdetached
+	    EtsiCAdESdetached,
+	    UnknownSignatureType ///< \since 0.90
 	};
 
 	/**
@@ -755,7 +765,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldSignature(DocumentData *doc, ::Page *p, ::FormWidgetSignature *w);
 	/// \endcond
-	~FormFieldSignature();
+	~FormFieldSignature() override;
 
 	FormType type() const override;
 
